<?php

declare (strict_types=1);
namespace WappoVendor\Achse\Math\Interval\SingleDayTime;

use WappoVendor\Achse\Comparable\IComparable;
use WappoVendor\Achse\Math\Interval\Boundary;
use WappoVendor\Achse\Math\Interval\DateTimeImmutable\DateTimeImmutable;
use WappoVendor\Achse\Math\Interval\DateTimeImmutable\DateTimeImmutableBoundary;
use WappoVendor\Achse\Math\Interval\DateTimeImmutable\DateTimeImmutableInterval;
use WappoVendor\Achse\Math\Interval\Interval;
use WappoVendor\Achse\Math\Interval\IntervalRangesInvalidException;
use WappoVendor\Achse\Math\Interval\ModificationNotPossibleException;
use DateTimeInterface;
final class SingleDayTimeInterval extends \WappoVendor\Achse\Math\Interval\Interval
{
    /**
     * @inheritdoc
     */
    public function __construct(\WappoVendor\Achse\Math\Interval\Boundary $left, \WappoVendor\Achse\Math\Interval\Boundary $right)
    {
        $this->validateBoundaryChecks($left, $right, \WappoVendor\Achse\Math\Interval\SingleDayTime\SingleDayTimeBoundary::class);
        if ($left->isGreaterThan($right) && !$this->isEndingAtMidnightNextDay($left, $right)) {
            throw new \WappoVendor\Achse\Math\Interval\IntervalRangesInvalidException('Left endpoint cannot be greater then Right endpoint.');
        }
        $this->left = $left;
        $this->right = $right;
        // intentionally, no parent constructor calling
    }
    /**
     * @inheritdoc
     */
    protected function isContainingElementRightCheck(\WappoVendor\Achse\Comparable\IComparable $element) : bool
    {
        return $this->isRightOpened() && $this->getRight()->getValue()->isGreaterThan($element) || $this->isRightClosed() && $this->getRight()->getValue()->isGreaterThanOrEqual($element) || $this->isEndingAtMidnightNextDay($this->left, $this->right);
    }
    /**
     * @param string $from
     * @param string $till
     * @return SingleDayTimeInterval
     */
    public static function fromString(string $from, string $till) : \WappoVendor\Achse\Math\Interval\SingleDayTime\SingleDayTimeInterval
    {
        return new static(new \WappoVendor\Achse\Math\Interval\SingleDayTime\SingleDayTimeBoundary(\WappoVendor\Achse\Math\Interval\SingleDayTime\SingleDayTime::from($from), \WappoVendor\Achse\Math\Interval\Boundary::CLOSED), new \WappoVendor\Achse\Math\Interval\SingleDayTime\SingleDayTimeBoundary(\WappoVendor\Achse\Math\Interval\SingleDayTime\SingleDayTime::from($till), \WappoVendor\Achse\Math\Interval\Boundary::OPENED));
    }
    /**
     * @param DateTimeImmutableInterval $interval
     * @param DateTimeInterface $date
     * @return static
     */
    public static function fromDateTimeInterval(\WappoVendor\Achse\Math\Interval\DateTimeImmutable\DateTimeImmutableInterval $interval, \DateTimeInterface $date) : \WappoVendor\Achse\Math\Interval\SingleDayTime\SingleDayTimeInterval
    {
        $thisDayInterval = self::buildWholeDayInterval($date);
        /** @var DateTimeImmutableInterval $intersection */
        $intersection = $thisDayInterval->intersection($interval);
        $left = \WappoVendor\Achse\Math\Interval\SingleDayTime\SingleDayTime::fromDateTime($intersection->getLeft()->getValue());
        $right = \WappoVendor\Achse\Math\Interval\SingleDayTime\SingleDayTime::fromDateTime($intersection->getRight()->getValue());
        return new static(new \WappoVendor\Achse\Math\Interval\SingleDayTime\SingleDayTimeBoundary($left, $intersection->getLeft()->getState()), new \WappoVendor\Achse\Math\Interval\SingleDayTime\SingleDayTimeBoundary($right, $intersection->getRight()->getState()));
    }
    /**
     * @param DateTimeInterface $date
     * @return DateTimeImmutableInterval
     */
    protected static function buildWholeDayInterval(\DateTimeInterface $date) : \WappoVendor\Achse\Math\Interval\DateTimeImmutable\DateTimeImmutableInterval
    {
        $start = \WappoVendor\Achse\Math\Interval\DateTimeImmutable\DateTimeImmutable::from($date)->setTime(0, 0, 0);
        $ends = \WappoVendor\Achse\Math\Interval\DateTimeImmutable\DateTimeImmutable::from($date)->setTime(23, 59, 59);
        $thisDayInterval = new \WappoVendor\Achse\Math\Interval\DateTimeImmutable\DateTimeImmutableInterval(new \WappoVendor\Achse\Math\Interval\DateTimeImmutable\DateTimeImmutableBoundary($start, \WappoVendor\Achse\Math\Interval\Boundary::CLOSED), new \WappoVendor\Achse\Math\Interval\DateTimeImmutable\DateTimeImmutableBoundary($ends, \WappoVendor\Achse\Math\Interval\Boundary::CLOSED));
        return $thisDayInterval;
    }
    /**
     * @param SingleDayTimeInterval $other
     * @param string $precision
     * @return bool
     */
    public function isFollowedByWithPrecision(\WappoVendor\Achse\Math\Interval\Interval $other, $precision) : bool
    {
        try {
            $this->getRight()->getValue()->modify('+' . $precision);
            $other->getLeft()->getValue()->modify('-' . $precision);
        } catch (\WappoVendor\Achse\Math\Interval\ModificationNotPossibleException $e) {
            return FALSE;
        }
        $dummyDay = new \WappoVendor\Achse\Math\Interval\DateTimeImmutable\DateTimeImmutable('2001-01-01 00:00:00');
        return $this->toDateTimeInterval($dummyDay)->isFollowedByWithPrecision($other->toDateTimeInterval($dummyDay), $precision);
    }
    /**
     * @return SingleDayTimeBoundary
     */
    public function getRight() : \WappoVendor\Achse\Math\Interval\Boundary
    {
        return parent::getRight();
    }
    /**
     * @return SingleDayTimeBoundary
     */
    public function getLeft() : \WappoVendor\Achse\Math\Interval\Boundary
    {
        return parent::getLeft();
    }
    /**
     * @param DateTimeInterface $day
     * @return DateTimeImmutableInterval
     */
    public function toDateTimeInterval(\DateTimeInterface $day) : \WappoVendor\Achse\Math\Interval\DateTimeImmutable\DateTimeImmutableInterval
    {
        $left = new \WappoVendor\Achse\Math\Interval\DateTimeImmutable\DateTimeImmutableBoundary($this->getLeft()->getValue()->toDateTime($day), $this->getLeft()->getState());
        $right = new \WappoVendor\Achse\Math\Interval\DateTimeImmutable\DateTimeImmutableBoundary($this->getRight()->getValue()->toDateTime($day), $this->getRight()->getState());
        return new \WappoVendor\Achse\Math\Interval\DateTimeImmutable\DateTimeImmutableInterval($left, $right);
    }
    /**
     * @param IComparable $element
     * @param bool $state
     * @return SingleDayTimeBoundary
     */
    protected function buildBoundary(\WappoVendor\Achse\Comparable\IComparable $element, bool $state) : \WappoVendor\Achse\Math\Interval\Boundary
    {
        return new \WappoVendor\Achse\Math\Interval\SingleDayTime\SingleDayTimeBoundary($element, $state);
    }
    /**
     * @param Boundary $left
     * @param Boundary $right
     * @return bool
     */
    private function isEndingAtMidnightNextDay(\WappoVendor\Achse\Math\Interval\Boundary $left, \WappoVendor\Achse\Math\Interval\Boundary $right) : bool
    {
        return !($left->isOpened() && $left->getValue()->isEqual($right->getValue())) && $right->isOpened() && $right->getValue()->isEqual($this->getZeroElement());
    }
    /**
     * @return SingleDayTime
     */
    private function getZeroElement() : \WappoVendor\Achse\Math\Interval\SingleDayTime\SingleDayTime
    {
        return new \WappoVendor\Achse\Math\Interval\SingleDayTime\SingleDayTime(0, 0, 0);
    }
}
